#!/usr/bin/perl -w

# chirp_seq.pl is a tool to analyse ChIRP-seq data of even and odd lanes
# script is written by Kun Qu in Stanford Dermatology

use strict;
use warnings FATAL => 'all';
use Getopt::Std;
use Getopt::Long;
use File::Basename;
use Cwd qw[abs_path];
use File::Spec;


my @usage;
push @usage, "Usage: ".basename($0)." [options]\n";
push @usage, "Run the chirp-seq pipeline for peak discovery.\n";
push @usage, "Command Line Example: perl chirp_seq.pl -c config.txt -o output.txt\n";
push @usage, "  -h, --help        Displays this information\n";
push @usage, "  -o, --output      Output Filename\n";
push @usage, "  -c, --config      Config Filename\n";


my $help;
my $output_filename;
my $config_filename;


GetOptions 
(
 'help'        => \$help,
 'output=s'    => \$output_filename,
 'config=s'    => \$config_filename,
);

not defined $help or die @usage;
defined $config_filename or die @usage;
defined $output_filename or die @usage;



#my $config = configdata->new();
my $config = new();
$config->read($config_filename);


# Config values
my $evenlane_filename  = $config->get_value("evenlane_filename");
my $oddlane_filename = $config->get_value("oddlane_filename");
my $inputlane_filename = $config->get_value("inputlane_filename");

my $scripts_directory = $config->get_value("scripts_directory");
my $chromosome_size_file = $config->get_value("chromosome_size_file");
my $raw_read_length = $config->get_value("raw_read_length");
my $band_width = $config->get_value("band_width");

my $wigToBigWig = $config->get_value("wigToBigWig");
my $bigWigToBedGraph = $config->get_value("bigWigToBedGraph");
my $bedGraphToBigWig = $config->get_value("bedGraphToBigWig");

my $bowtie_bin = $config->get_value("bowtie_bin");
my $bowtie_index = $config->get_value("bowtie_index");
my $bowtie_threads = $config->get_value("bowtie_threads");
my $macs_bin = $config->get_value("macs_bin");



# scripts
my $sam2bedGrpah_script = "$scripts_directory/sam2bedGraph_norm.py";
my $merge2bedGraph_script = "$scripts_directory/combine_two_bedGraph.pl";
my $bedGraph2sam_script = "$scripts_directory/bedGraph2sam.pl";
my $peak_correlation_script = "$scripts_directory/peak_correlation.pl";


my $mailto;
if ($config->has_value("mailto")) {
  $mailto = $config->get_value("mailto");
}


if (!$evenlane_filename) {
  die "Error: Even lane missing in config file $config_filename\n";
}

if (!$oddlane_filename) {
  die "Error: Odd lane missing in config file $config_filename\n";
}



print "Align even lane reads using bowtie\n";
system "$bowtie_bin -m 1 -p $bowtie_threads -S $bowtie_index $evenlane_filename even.sam";

print "Align odd lane reads using bowtie\n";
system "$bowtie_bin -m 1 -p $bowtie_threads -S $bowtie_index $oddlane_filename odd.sam";

if ($inputlane_filename) {
  print "Align input lane reads using bowtie\n";
  system "$bowtie_bin -m 1 -p $bowtie_threads -S $bowtie_index $inputlane_filename input.sam";
}




print "Converte .sam to .bedGraph file for even lane\n";
system "$sam2bedGrpah_script even.sam even.bedGraph";

print "Converte .sam to .bedGraph file for odd lane\n";
system "$sam2bedGrpah_script odd.sam odd.bedGraph";

if ($inputlane_filename) {
  print "Converte .sam to .bedGraph file for input lane\n";
  system "$sam2bedGrpah_script input.sam input.bedGraph";
}



print "merge even odd lanes\n";
system "$merge2bedGraph_script $raw_read_length $chromosome_size_file even.bedGraph odd.bedGraph merge.wig";



print "convert .wig file to .sam file\n";
system "$wigToBigWig merge.wig $chromosome_size_file merge.bw";
system "$bigWigToBedGraph merge.bw merge.bedGraph";
system "$bedGraph2sam_script $chromosome_size_file merge.bedGraph merge.sam";


print "convert .bedGraph files to .bw files\n";
system "$bedGraphToBigWig even.bedGraph $chromosome_size_file even.bw";
system "$bedGraphToBigWig odd.bedGraph $chromosome_size_file odd.bw";
system "$bedGraphToBigWig input.bedGraph $chromosome_size_file input.bw";


print "call peaks using macs\n";
system "$macs_bin -t merge.sam -c input.sam -f SAM -n merge --bw $band_width -m 10,50";



print "calculate correlation of macs predicted peaks between even and odd lanes\n";
system "$peak_correlation_script merge_peaks.xls even.bedGraph odd.bedGraph merge.bedGraph";
system "cp merge_peaks.xls.corr.xls $output_filename";


mailme();




sub mailme {
  return if not defined $mailto;
  my $text = "ChIRP-seq analysis finished";
  print "Attempting to mail $mailto the complete notice\n";
  system "echo '$text' | mail -s 'ChIRP-seq complete notice' $mailto";
}

sub new {
  my $self = {};
  bless($self);
  return $self;
}

sub read {
  my $self = shift;
  my $config_filename = shift;
  my %config_values;
  
  open CFG, $config_filename or die "Error: Unable to open $config_filename\n";
  while (<CFG>)
    {
      chomp;
      /^\s*([^=\s]+)\s*=\s*(.*)$/;
      
      my $key = $1;
      my $value = $2;
      
      next if not defined $key;
      next if not defined $value;
      
      $config_values{$key} = $value;
    }
  close CFG;
  
  foreach my $key (keys %config_values)
    {
      while ($config_values{$key} =~ /\$\(([^)]+)\)/)
	{
	  my $other_key = $1;
	  
	  if (not defined $config_values{$other_key})
	    {
	      die "Error: no value for $other_key in config file $config_filename\n";
	    }
	  
	  $config_values{$key} =~ s/\$\($other_key\)/$config_values{$other_key}/;
	}
    }
  
  $self->{"config_values"} = \%config_values;
  $self->{"config_filename"} = $config_filename;
}

sub has_value {
  my $self = shift;
  my $key = shift;
  
  my $config_values = $self->{"config_values"};
  my $config_filename = $self->{"config_filename"};
  
  defined $config_values and defined $config_filename or die "Error: config not read\n";
  
  return defined $config_values->{$key};
}

sub get_value {
  my $self = shift;
  my $key = shift;
  
  my $config_values = $self->{"config_values"};
  my $config_filename = $self->{"config_filename"};
  
  defined $config_values and defined $config_filename or die "Error: config not read\n";
    
  return $config_values->{$key};
}

sub get_list {
  my $self = shift;
  my $key = shift;
  
  my $index = 1;
  my $key_index = $key.$index;
  my @values;
  while ($self->has_value($key_index))
    {
      push @values, $self->get_value($key_index);
      $index++;
      $key_index = $key.$index;
    }
  
  return @values;
}

sub get_hash {
  my $self = shift;
  my $key = shift;
  
  my $config_values = $self->{"config_values"};
  my $config_filename = $self->{"config_filename"};
  
  defined $config_values and defined $config_filename or die "Error: config not read\n";
  
  if (not defined $config_values->{$key})
    {
      die "Error: no value for $key in config file $config_filename\n";
    }
  
  my %values;
  foreach my $value (split /,/, $config_values->{$key})
    {
      $value =~ s/\s//g;
      $values{$value} = 1;
    }
  
  return %values;
}



